@@ -18,15 +18,16 @@ from tkMessageBox import showerror, showinfo
 
 from wurfl import *
 from pywurfl.exceptions import *
-from pywurfl.algorithms import Tokenizer
+from pywurfl.algorithms import Tokenizer, JaroWinkler, TwoStepAnalysis
 
 try:
     from pywurfl.algorithms import JaroWinkler
     JAROWINKLER = True
-    jsearch = JaroWinkler(accuracy=.85)
 except ImportError:
     JAROWINKLER = False
 
+tssearch = TwoStepAnalysis(devices)
+jsearch = JaroWinkler(accuracy=.85)
 tsearch = Tokenizer()
 
 def get_info():
@@ -36,7 +37,6 @@ def get_info():
     if type_var.get() == "User Agent":
         try:
             device = devices.select_ua(ua, actual_device_root=adr_var.get(),
-                                       normalize=normalize_var.get(),
                                        search=get_algo())
         except DeviceNotFound:
             showerror('Error', "Device Not Found")
@@ -54,10 +54,13 @@ def get_info():
     display_device_info(device)
 
 def get_algo():
-    if JAROWINKLER:
-        if stype_var.get() == "Jaro-Winkler":
-            return jsearch
-    return tsearch
+    if stype_var.get() == "Two Step Analysis":
+        search =  tssearch
+    elif stype_var.get() == "Jaro-Winkler":
+        search = jsearch
+    else:
+        search = tsearch
+    return search
 
 def follow_link(event):
     indx = infotext.index("@%d, %d" % (event.x, event.y))
@@ -73,7 +76,6 @@ def follow_link(event):
     display_device_info(device)
 
 def display_device_info(device):
-
     infotext.config(state=NORMAL)
     infotext.delete("1.0", END)
     infotext.insert(END, device)
@@ -112,7 +114,7 @@ def do_group_markup():
 
 def do_link_markup():
     start, finish = infotext.tag_ranges('fallbacks')
-    f1, f2 = finish.split('.')
+    f1, f2 = str(finish).split('.')
     f1 = int(f1)
     f2 = int(f2)
 
@@ -172,17 +174,16 @@ def change_type():
 
 
 def about_pywurfl():
-    msg = "pywurfl Browser Example\nCopyright 2004-2006, Armand Lynch "
+    msg = "pywurfl Browser Example\nCopyright 2004-2010, Armand Lynch "
     msg += "<lyncha@users.sourceforge.net>\n\n"
     msg += "This code is free software; you can redistribute it and/or\n"
-    msg += "modify it under the terms of the BSD License.\n"
+    msg += "modify it under the terms of the LGPL License.\n"
 
     showinfo("About pywurfl Browser", msg)
 
 
-
 root = Tk()
-root.title("pywurfl Browser v5.1")
+root.title("pywurfl Browser v6.0")
 
 mroot = Menu(root)
 
@@ -191,16 +192,10 @@ mfile.add_command(label="Exit", command=root.quit)
 
 type_var = StringVar()
 stype_var = StringVar()
-normalize_var = BooleanVar()
 adr_var = BooleanVar()
 
 type_var.set("User Agent")
-if JAROWINKLER:
-    stype_var.set("Jaro-Winkler")
-else:
-    stype_var.set("Tokenizer")
-# normalize_noise
-normalize_var.set(True)
+stype_var.set("Two Step Analysis")
 # actual_device_root
 adr_var.set(False)
 
@@ -210,12 +205,11 @@ soptions.add_radiobutton(label="User Agent", variable=type_var,
 soptions.add_radiobutton(label="WURFL ID", variable=type_var,
                          command=change_type)
 soptions.add_separator()
-soptions.add_checkbutton(label="Normalize User Agent", variable=normalize_var)
 soptions.add_checkbutton(label="Actual Device", variable=adr_var)
-if JAROWINKLER:
-    soptions.add_separator()
-    soptions.add_radiobutton(label="Jaro-Winkler", variable=stype_var)
-    soptions.add_radiobutton(label="Tokenizer", variable=stype_var)
+soptions.add_separator()
+soptions.add_radiobutton(label="Two Step Analysis", variable=stype_var)
+soptions.add_radiobutton(label="Jaro-Winkler", variable=stype_var)
+soptions.add_radiobutton(label="Tokenizer", variable=stype_var)
 
 mabout = Menu(mroot, tearoff=0)
 mabout.add_command(label="About", command=about_pywurfl)
@@ -244,7 +238,7 @@ infotext.tag_config("group", borderwidth=1, foreground="#fdcd00",
                     font=("Helvetica", "11", "bold"))
 infotext.tag_config("meta", borderwidth=1, foreground="#778899",
                     font=("Helvetica", "11", "bold"))
-infotext.tag_config("fallbacks", borderwidth=1, foreground="#778899",
+infotext.tag_config("fallbacks", borderwidth=1, foreground="#998877",
                     font=("Helvetica", "11", "bold"), wrap="word")
 infotext.tag_config("false", foreground="red")
 infotext.tag_config("true", foreground="darkgreen")
