@@ -23,12 +23,13 @@ pywurfl - Python tools for processing and querying the Wireless Universal Resour
 """
 
 import re
-import md5
+import hashlib
+import warnings
 from copy import copy
 
 from pywurfl.exceptions import (WURFLException, ActualDeviceRootNotFound,
                                 DeviceNotFound, ExistsException)
-from pywurfl.algorithms.wurfl.normalizers import default_normalizer
+from pywurfl.algorithms.wurfl import normalizers
 
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
@@ -36,7 +37,7 @@ __contributors__ = "Pau Aliagas <pau@newtral.org>"
 __copyright__ = "Copyright 2006-2010, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
-__version__ = "7.1.1"
+__version__ = "7.2"
 __all__ = ['devclass', 'Devices']
 
 
@@ -141,7 +142,7 @@ class Devices(object):
             device = device.__bases__[0]
         raise ActualDeviceRootNotFound
 
-    def select_ua(self, devua, actual_device_root=False, normalize=True,
+    def select_ua(self, devua, actual_device_root=False, normalize=None,
                   search=None, instance=True):
         """
         Return a Device object based on the user agent.
@@ -151,8 +152,8 @@ class Devices(object):
         @param actual_device_root: Return a device that is an actual device
                                    root
         @type actual_device_root: boolean
-        @param normalize: Normalize the user agent by removing extraneous text.
-        @type normalize: boolean
+        @param normalize: Deprecated
+        @type normalize: None
         @param search: The algorithm to use for searching. If 'search' is None,
                        a search will not be performed.
         @type search: pywurfl.Algorithm
@@ -162,28 +163,36 @@ class Devices(object):
         @raise DeviceNotFound:
         """
         _unicode_check(u'devua', devua)
+
+        if normalize is not None:
+            warnings.warn("use of the `normalize` argument in select_ua is "
+                          "deprecated", DeprecationWarning)
+
+        if search is not None and not hasattr(search, "use_normalized_ua"):
+            warnings.warn("search algorithm object should have a "
+                          "`use_normalized_ua` boolean attribute.",
+                          FutureWarning)
+            # AJL Thu Dec 16 13:35:37 EST 2010
+            # We'll assume that the search algorithm would require a normalized
+            # user agent.
+            search.use_normalized_ua = True
+
         devua = devua.strip()
-        device = None
-        if devua in self.devuas:
-            device = self.devuas.get(devua)
-            if actual_device_root:
-                device = self.find_actual_root(device)
+        device = self.devuas.get(devua)
 
         if device is None:
-            if normalize:
-                devua = default_normalizer(devua)
-
-                if devua in self.devuas:
-                    device = self.devuas.get(devua)
-                    if actual_device_root:
-                        device = self.find_actual_root(device)
+            norm_devua = normalizers.generic(devua)
+            device = self.devuas.get(norm_devua)
 
             if device is None and search is not None:
-                device = search(devua, self)
-                if actual_device_root:
-                    device = self.find_actual_root(device)
+                if search.use_normalized_ua:
+                    device = search(norm_devua, self)
+                else:
+                    device = search(devua, self)
 
         if device is not None:
+            if actual_device_root:
+                device = self.find_actual_root(device)
             if instance:
                 return device()
             else:
@@ -521,9 +530,8 @@ class Devices(object):
         """
         Return MD5 hex digest for all WURFL data
         """
-        data = []
-        [data.append(str(self.devids[x]())) for x in sorted(self.devids)]
-        return md5.new(u''.join(data)).hexdigest()
+        data = (str(self.devids[x]()) for x in sorted(self.devids))
+        return hashlib.md5(u''.join(data)).hexdigest()
 
     def __iter__(self):
         return self.devids.__iter__()
