@@ -26,7 +26,7 @@ A description of the way the following source is intended to work can be found
 within the source for the original Java API implementation here:
 http://sourceforge.net/projects/wurfl/files/WURFL Java API/
 
-The original Java code is GPLd and Copyright (c) 2008-2009 WURFL-Pro srl
+The original Java code is GPLd and Copyright (c) WURFL-Pro srl
 """
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
@@ -42,14 +42,13 @@ def ris_match(candidates, needle, tolerance):
 	# given needle. If there is not candidates within given tolerance, it
 	# returns the empty string.
     match = u""
-    best_match = u""
     needle_length = len(needle)
 
     best_distance = -1
+    best_match_index = -1
     low = 0
     high = len(candidates) - 1
 
-
     # Perfect match: best_distance == needle length()
     # Best match: best_distance >= tolerance
     # First match: best_distance == tolerance
@@ -58,10 +57,10 @@ def ris_match(candidates, needle, tolerance):
         mid = (low + high) / 2
         mid_candidate = candidates[mid]
 
-        distance = get_ris_distance(needle, mid_candidate)
+        distance = longest_common_prefix(needle, mid_candidate)
 
         if distance > best_distance:
-            best_match = mid_candidate
+            best_match_index = mid
             best_distance = distance
 
         # Calculate low and high
@@ -74,12 +73,13 @@ def ris_match(candidates, needle, tolerance):
 
     # Candidate is found
     if best_distance >= tolerance:
-        match = best_match
+        match = first_of_best_matches(needle, candidates, best_match_index,
+                                      best_distance)
 
     return match
 
 
-def get_ris_distance(t1, t2):
+def longest_common_prefix(t1, t2):
     i = 0
     t = min(len(t1), len(t2))
 
@@ -91,6 +91,16 @@ def get_ris_distance(t1, t2):
     return i
 
 
+def first_of_best_matches(needle, candidates, best_match_index, best_distance):
+    best_match = candidates[best_match_index]
+    for candidate in reversed(candidates[:best_match_index-1]):
+        if best_distance == longest_common_prefix(needle, candidate):
+            best_match = candidate
+        else:
+            break
+    return best_match
+
+
 def ld_match(candidates, needle, tolerance):
     needle_length = len(needle)
     user_agent = u""
