@@ -26,7 +26,7 @@ A description of the way the following source is intended to work can be found
 within the source for the original Java API implementation here:
 http://sourceforge.net/projects/wurfl/files/WURFL Java API/
 
-The original Java code is GPLd and Copyright (c) 2008-2009 WURFL-Pro srl
+The original Java code is GPLd and Copyright (c) WURFL-Pro srl
 """
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
@@ -37,17 +37,37 @@ __version__ = "1.0.1"
 
 from functools import partial
 
+mobile_browsers = [u"cldc", u"symbian", u"midp", u"j2me", u"mobile",
+                   u"wireless", u"palm", u"phone", u"pocket pc",
+                   u"pocketpc", u"netfront", u"bolt", u"iris", u"brew",
+                   u"openwave", u"windows ce", u"wap2", u"android",
+                   u"opera mini", u"opera mobi", u"maemo", u"fennec",
+                   u"blazer", u"160x160", u"tablet", u"webos", u"sony",
+                   u"nintendo", u"480x640", u"aspen simulator",
+                   u"up.browser", u"up.link", u"embider", u"danger hiptop",
+                   u"obigo", u"foma"]
+
+desktop_browsers = [u"slcc1", u".net clr", u"trident/4", u"media center pc",
+                    u"funwebproducts", u"macintosh", u"wow64", u"aol 9.",
+                    u"america online browser", u"googletoolbar"]
+
+
+def is_typeof_browser(user_agent, browsers=None):
+    if browsers is None:
+        return False
 
-def is_mobile_browser(user_agent):
     user_agent = user_agent.lower()
-    mobile_browsers = [u"tablet", u"mobile", u"wireless", u"palm", u"blazer",
-                       u"netfront", u"symbian", u"bolt", u"iris", u"pocketpc"]
-    for mb in mobile_browsers:
-        if mb in user_agent:
+
+    for b in browsers:
+        if b in user_agent:
             return True
     return False
 
 
+is_mobile_browser = partial(is_typeof_browser, browsers=mobile_browsers)
+is_desktop_browser = partial(is_typeof_browser, browsers=desktop_browsers)
+
+
 def ordinal_index(target, needle=u" ", ordinal=1, start_index=0):
     index = -1
 
@@ -85,5 +105,6 @@ first_space = indexof_or_length
 first_slash = partial(indexof_or_length, needle=u"/")
 second_slash = partial(indexof_or_length, needle=u"/", position=2)
 first_semi_colon = partial(indexof_or_length, needle=u";")
+third_space = partial(indexof_or_length, position=3)
 
 
