@@ -380,7 +380,7 @@ def get_operators():
 ops = get_operators()
 
 
-def expr_test(lexpr, op, rexpr):
+def expr_test_func(lexpr, op, rexpr):
     """
     Returns an exp test function.
 
@@ -551,11 +551,11 @@ def reduce_statement(exp):
     funcs = []
     if exp.any_statement:
         for expr in exp.any_statement.any_expr_list:
-            funcs.append(expr_test(expr, exp.operator, exp.rexpr))
+            funcs.append(expr_test_func(expr, exp.operator, exp.rexpr))
         func = ops['or']
     elif exp.all_statement:
         for expr in exp.all_statement.all_expr_list:
-            funcs.append(expr_test(expr, exp.operator, exp.rexpr))
+            funcs.append(expr_test_func(expr, exp.operator, exp.rexpr))
         func = ops['and']
     return reduce_funcs(func, funcs)
 
@@ -588,8 +588,8 @@ def test_generator(ql_result):
         if where_test.any_statement or where_test.all_statement:
             func = reduce_statement(where_test)
         else:
-            func = expr_test(where_test.lexpr, where_test.operator,
-                             where_test.rexpr)
+            func = expr_test_func(where_test.lexpr, where_test.operator,
+                                  where_test.rexpr)
 
         boolop = where_test.boolop
         if boolop:
