@@ -1,5 +1,5 @@
 # pywurfl - Wireless Universal Resource File Tools in Python
-# Copyright (C) 2006-2009 Armand Lynch
+# Copyright (C) 2006-2010 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -23,21 +23,14 @@ pywurfl WURFL class hierarchy serializer
 """
 
 from time import ctime
-
-try:
-    from xml.etree.ElementTree import ElementTree, Element, SubElement
-except ImportError:
-    try:
-        from cElementTree import ElementTree, Element, SubElement
-    except ImportError:
-        from elementtree.ElementTree import ElementTree, Element, SubElement
+from xml.etree.ElementTree import ElementTree, Element, SubElement
 
 from pywurfl import __version__ as pw_version
 from pywurfl.exceptions import WURFLException
 
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
-__copyright__ = "Copyright 2006-2009, Armand Lynch"
+__copyright__ = "Copyright 2006-2010, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
 __all__ = ['Serialize']
