@@ -25,14 +25,7 @@ WURFL processing utility.
 
 import sys
 from optparse import OptionParser
-
-try:
-    from xml.etree.ElementTree import parse
-except ImportError:
-    try:
-        from cElementTree import parse
-    except ImportError:
-        from elementtree.ElementTree import parse
+from xml.etree.ElementTree import parse
 
 from pywurfl.exceptions import WURFLException
 
